import 'dart:async';
import 'package:build/build.dart';
import "package:jaguar_generator_config/jaguar_generator_config.dart";
import 'package:jaguar_http_cli/jaguar_http_cli.dart';
import 'package:source_gen/source_gen.dart';
import 'package:build_runner/build_runner.dart' as build_runner;

const _defaultPath = const [
  "lib/**/**.dart",
  "bin/**/**.dart",
  "test/**/**.dart",
  "example/**/**.dart",
  "lib/*.dart",
  "bin/*.dart",
  "test/*.dart",
  "example/*.dart"
];

const jaguarHttpConfigFile = "pubspec.yaml";

class JaguarHttpConfig extends GeneratorConfig {
  static const String httpKey = 'jaguar_http';

  JaguarHttpConfig({String configFileName: jaguarHttpConfigFile})
      : super(configFileName: configFileName);

  List<String> get httpFiles => config[httpKey];
}

/// Watch files and trigger build function
Future<build_runner.ServeHandler> watch() =>
    build_runner.watch(buildActions(), deleteFilesByDefault: true);

/// Build all Http Class
Future<build_runner.BuildResult> build() =>
    build_runner.build(buildActions(), deleteFilesByDefault: true);

List<build_runner.BuildAction> buildActions(
    {String configFileName: jaguarHttpConfigFile, String copyrightContent}) {
  final config = new JaguarHttpConfig();

  if (config.pubspec.projectName == null) {
    throw "Could not find the project name";
  }

  var httpFiles = config.httpFiles;

  if (httpFiles == null || httpFiles.isEmpty) {
    print(
        "[WARNING] Jaguar Http did not find any files to watch in 'pubspec.yaml', '$_defaultPath' used by default");
    httpFiles = _defaultPath;
  }

  return [
    new build_runner.BuildAction(
        jaguarHttpPartBuilder(header: copyrightContent),
        config.pubspec.projectName,
        inputs: config.httpFiles),
  ];
}

Builder jaguarHttpPartBuilder({String header}) =>
    new PartBuilder([const JaguarHttpGenerator()], header: header);
